@@ -2,6 +2,7 @@ import sys, os
 from time import gmtime, strftime
 import numpy as np
 import logging
+from numpy import random
 
 file_handler = logging.FileHandler(filename='nustar_utils_%s.log' % (strftime("%Y-%m-%dT%H:%M:%S", gmtime())))
 stdout_handler = logging.StreamHandler(sys.stdout)
@@ -383,6 +384,20 @@ def make_tphase(freq,  min_en=3., max_en=70., t_step=1000, n_bins=32, orbitfile=
             run()
 
 def rebin_matrix(e_min, e_max, pp, dpp, min_s_n = 50, only_pulsed=False):
+    '''
+
+    :param e_min: array with minimum energy of each bin
+    :param e_max: array with maximum energy of each bin
+    :param pp: 2-d array with pulse profiles
+    :param dpp: 2-d array with pulse profile uncertainties
+    :param min_s_n: minimum S/N for rebin
+    :param only_pulsed:  it subtracts the average value before rebinning
+    :return:
+    new_e_mins rebinned array with minimum energy of each bin
+    new_e_maxs rebinned array with maximum energy of each bin
+    new_pulses rebinned 2-d array with pulse profiles
+    dnew_pulses rebinned 2-d array with pulse profile uncertainties
+    '''
 
     new_pulses = []
     dnew_pulses = []
@@ -420,25 +435,8 @@ def rebin_matrix(e_min, e_max, pp, dpp, min_s_n = 50, only_pulsed=False):
                 i1 = i2 - 1
 
     logger.info('We rebinned from %d to %d bins at a minimum S/N of %.1f' % (len(e_min), len(new_e_mins), min_s_n))
-    #print('We rebinned from %d to %d bins at a minimum S/N of %.1f' % (len(e_min), len(new_e_mins), min_s_n))
-
     return np.array(new_e_mins), np.array(new_e_maxs), np.array(new_pulses), np.array(dnew_pulses)
 
-
-import random
-
-
-def fake_profile(conteggi, errore):
-    random_pulse = []
-
-    for i in range(len(conteggi)):
-        # print(conteggi[i],errore[i])
-        pulse = random.gauss(conteggi[i], errore[i])
-        random_pulse.append(pulse)
-
-    return np.array(random_pulse)
-
-
 def pulse_fraction_from_data_rms(counts, counts_err, n_harm=3):
     a0 = np.mean(counts)
     K = n_harm
@@ -499,40 +497,65 @@ def get_error_from_simul_rms(counts, counts_err, n_simul=100, n_harm=3):
     return get_error_from_simul(counts, counts_err, pulse_fraction_from_data_rms,
                                 n_simul=n_simul, n_harms=n_harm)
 
-def get_error_from_simul(counts, counts_err, method, n_simul=100, **kwargs):
+def get_error_from_simul(counts, counts_err, method, n_simul=100, use_poisson=False, **kwargs):
+    '''
+
+    :param counts: input data
+    :param counts_err: input data error
+    :param method: a fnction with the method to compute the quantity
+    :param n_simul: number of simulations
+    :param kwargs: additional arguments to the function to be computed
+    :return:
+    '''
     simul_rms = []
     for i in range(n_simul):
-        fp = fake_profile(counts, counts_err)
+        if use_poisson:
+            fp = random.poisson(counts)
+        else:
+            fp = random.normal(counts, counts_err)
         simul_rms.append(method(fp, counts_err, **kwargs))
 
     return np.std(simul_rms)
 
-def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, plot=False, verbose=True):
+def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, n_harm_max=-1, plot=False, verbose=True, label=''):
     '''
-    Computes the pulsed fraction using an FFT. It stops as soon as the pulse is described at beter than level
-    :param x:
-    :param dx:
-    :param level:
-    :param n_harm_min:
-    :param plot:
-    :return: pulsed fraction
+    Computes the pulsed fraction using an FFT. It stops as soon as the pulse is described at better than level
+    :param x: pulse profile
+    :param dx: pulse profile uncertainty
+    :param level: confidenvce level for chi^2 test to stop number of harmonics (default 0.1)
+    :param n_harm_min: minimum number of harmonics to use (default 2)
+    :param n_harm_max: minimum number of harmonics to use (default -1 takes the size of pulse profile)
+    :param plot: plot the pulse profile
+    :param lavel: to save the plot with name "rms_`label`.pdf", if label=='' it does not save the plot
+    :return: pulsed fraction (float)
     '''
     # import scipy.stats.chisquare as chi2
     from scipy.stats import chi2
     n = len(x)
 
-    fft = np.fft.fft(x)
+    if n_harm_max < n_harm_min or n_harm_max > n/2:
+        n_harm_max = n / 2
 
-    for n_harm in range(n_harm_min, int(np.floor(n / 2))):
+    fft = np.fft.fft(x)
+    old_chi2_sf = -1.0
+    for n_harm in range(int(n_harm_min), int(n_harm_max)):
         mask = np.ones(n, dtype=np.cdouble)
         mask[n_harm:-(n_harm - 1)] = 0 + 0j
         ifft = np.fft.ifft(fft * mask)
         y = np.real(ifft)
-
-        chi2_val = np.sum(((x - y) / dx) ** 2)
+        #Necessary to avoid infinite for zero uncertaity
+        ind = dx > 0
+        chi2_val = np.sum(((x[ind] - y[ind]) / dx[ind]) ** 2)
         dof = max(1, n - (1 + 2 * (n_harm - 1)))
-        if chi2.sf(chi2_val, dof) > level:
+        chi2_sf = chi2.sf(chi2_val, dof)
+        # print(chi2_val, dof)
+        # print(chi2_sf, old_chi2_sf)
+        #Sometimes, we do ot reach the required level, but we cannot describe the pulse significantly better,
+        # so we stop in any case (condition chi2_sf < old_chi2_sf)
+        if chi2_sf > level or chi2_sf < old_chi2_sf:
             break
+        old_chi2_sf = chi2_sf
+
     if verbose:
         logger.info("Used %d harmonics for pulse description" % n_harm)
     a = np.absolute(fft) / n
@@ -542,8 +565,12 @@ def fft_pulsed_fraction(x, dx, level=0.1, n_harm_min=2, plot=False, verbose=True
         import matplotlib.pyplot as plt
         plt.figure()
         f = np.linspace(0, 1, n)
-        plt.errorbar(f, x, yerr=dx, marker='.')
-        plt.plot(f, y)
+        plt.errorbar(f, x, yerr=dx, marker='.', label='data')
+        plt.plot(f, y, label='%d harmonics' % n_harm)
+        plt.xlabel('Phase')
+        plt.legend()
+        if label != '':
+            plt.savefig('rms_%s.pdf' % label)
 
     return pulsed_frac
 
@@ -713,10 +740,9 @@ def get_fourier_coeff(pulse):
 
     phi0 = np.arctan2(i_s, i_c) / 2 / np.pi
     phi0_2 = np.arctan2(i_s2, i_c2) / 2 / np.pi
-    A = np.sqrt(i_c * i_c + i_s * i_s) / len(pulse) / np.mean(pulse)
-
-    A2 = np.sqrt(i_c2 * i_c2 + i_s2 * i_s2) / len(pulse) / np.mean(pulse)
-    return phi0, phi0_2, A, A2
+    a = np.sqrt(i_c * i_c + i_s * i_s) / len(pulse) / np.mean(pulse)
+    a2 = np.sqrt(i_c2 * i_c2 + i_s2 * i_s2) / len(pulse) / np.mean(pulse)
+    return phi0, phi0_2, a, a2
 
 def get_fourier_coeff_error( counts, counts_err, n_simul=100):
 
@@ -726,7 +752,7 @@ def get_fourier_coeff_error( counts, counts_err, n_simul=100):
     sim_A2 = []
 
     for i in range(n_simul):
-        fp = fake_profile(counts, counts_err)
+        fp = random.gauss(counts, counts_err)
         phi0, phi0_2, A, A2 = get_fourier_coeff(fp)
         sim_phi0.append(phi0)
         sim_phi0_2.append(phi0_2)
@@ -735,6 +761,58 @@ def get_fourier_coeff_error( counts, counts_err, n_simul=100):
 
     return np.std(sim_phi0), np.std(sim_phi0_2,), np.std(sim_A), np.std(sim_A2)
 
+def write_orbit_file_from_gbm_page(url, file_name='orbit.dat',
+                                   orbital_parameters = ['axsin', 'Eccentricity', 'periastron',
+                                                         'T<sub>', 'Orbital Period', 'Period Deriv.']):
+    '''
+    It writes a file to be used in timingsuite for the orbit starting from the GBM page
+    This is tested just for Cen X-3 at the moment and it is very fragile
+
+    :param url: The url of the GBM page of the source
+        (e.g. 'https://gammaray.nsstc.nasa.gov/gbm/science/pulsars/lightcurves/cenx3.html')
+    :param file_name: the name of output file default 'orbit.dat'
+    :param orbital_parameters the orbital parameters to be extracted from the table
+        default: ['axsin', 'Eccentricity', 'periastron',
+                 'T<sub>', 'Orbital Period', 'Period Deriv.']
+    :return: None
+    '''
+    import requests
+    from bs4 import BeautifulSoup
+    import re
+    from astropy.time import Time
+
+    #Parses the table with a particular argument (fragile)
+    html = requests.get(url).content
+    soup = BeautifulSoup(html)
+    orbit_table = soup.find('table', attrs={'border': 2})
+    trs = orbit_table.find_all('tr')
+
+    #removes the first row as it contains the table title
+    trs[0].extract()
+
+    #Find numbers (https://stackoverflow.com/questions/4703390/how-to-extract-a-floating-number-from-a-string)
+    numeric_const_pattern = '[-+]? (?: (?: \d* \. \d+ ) | (?: \d+ \.? ) )(?: [Ee] [+-]? \d+ ) ?'
+    rx = re.compile(numeric_const_pattern, re.VERBOSE)
+
+    #for each parmeter it extracts the value as string
+    orbit = {}
+    for tr in trs:
+        tds = tr.find_all('td')
+        if len(tds) < 2:
+            continue
+        for k in orbital_parameters:
+            if k.lower() in str(tds[0]).lower():
+                orbit.update({k: rx.findall(str(tds[1]))[0]})
+
+    # it writes the orbit file
+    with open(file_name, 'w') as ff:
+        for kk in orbital_parameters:
+            if 'T' in kk:
+                t_90 = Time(orbit[kk], format='jd')
+                ff.write('%f\n' % (t_90.mjd - 0.25 * float(orbit['Orbital Period'])))
+            else:
+                ff.write(orbit[kk] + '\n')
+
 def get_cross_correlation(pp, plot=False):
 
     correlation = []
@@ -761,6 +839,17 @@ def get_cross_correlation(pp, plot=False):
     lag[lag > (pp.shape[1]/2)] -= pp.shape[1]
     return lag, np.array(correlation)
 
+def get_cross_correlation_error(pp, dpp, n_simul=100):
+
+    fake_lags = np.empty([n_simul, pp.shape[0]], dtype=float)
+    fake_corrs = np.empty([n_simul, pp.shape[0]], dtype=float)
+
+    for i in range(n_simul):
+        fake_pp = random.gauss(pp, dpp)
+        fake_lags[i, :], fake_corrs[i, :] = get_cross_correlation(fake_pp)
+
+    return np.std(fake_lags, 0), np.std(fake_corrs, 0)
+
 def get_target_coords_extern(input_name):
     from astroquery.simbad import Simbad
     from astropy import units as u
